@@ -40,6 +40,7 @@ local N = 'rbl'
 
 -- Checks that could be performed by rbl module
 local local_exclusions
+local disabled_rbl_suffixes -- Map of disabled rbl suffixes
 local white_symbols = {}
 local black_symbols = {}
 local monitored_addresses = {}
@@ -151,9 +152,7 @@ local function gen_check_rcvd_conditions(rbl, received_total)
     end
     if ((rh.real_ip:get_version() == 6 and rbl.ipv6) or
         (rh.real_ip:get_version() == 4 and rbl.ipv4)) and
-        ((rbl.exclude_private_ips and not rh.real_ip:is_local()) or
-            not rbl.exclude_private_ips) and ((rbl.exclude_local_ips and
-        not is_excluded_ip(rh.real_ip)) or not rbl.exclude_local_ips) then
+        ((rbl.exclude_local and not rh.real_ip:is_local() or is_excluded_ip(rh.real_ip)) or not rbl.exclude_local) then
       return true
     else
       return false
@@ -215,7 +214,45 @@ local function gen_check_rcvd_conditions(rbl, received_total)
   end
 end
 
-local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_table_elt)
+local matchers = {}
+
+matchers.radix = function(_, _, real_ip, map)
+  return map and map:get_key(real_ip) or false
+end
+
+matchers.equality = function(codes, to_match)
+  if type(codes) ~= 'table' then
+    return codes == to_match
+  end
+  for _, ip in ipairs(codes) do
+    if to_match == ip then
+      return true
+    end
+  end
+  return false
+end
+
+matchers.luapattern = function(codes, to_match)
+  if type(codes) ~= 'table' then
+    return string.find(to_match, '^' .. codes .. '$') and true or false
+  end
+  for _, pattern in ipairs(codes) do
+    if string.find(to_match, '^' .. pattern .. '$') then
+      return true
+    end
+  end
+  return false
+end
+
+matchers.regexp = function(_, to_match, _, map)
+  return map and map:get_key(to_match) or false
+end
+
+matchers.glob = function(_, to_match, _, map)
+  return map and map:get_key(to_match) or false
+end
+
+local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_table_elt, match)
   local function make_option(ip, label)
     if ip then
       return string.format('%s:%s:%s',
@@ -274,6 +311,8 @@ local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_tabl
     return
   end
 
+  local returncodes_maps = rbl.returncodes_maps or {}
+
   for _, result in ipairs(results) do
     local ipstr = result:to_string()
     lua_util.debugm(N, task, '%s DNS result %s', to_resolve, ipstr)
@@ -292,12 +331,10 @@ local function rbl_dns_process(task, rbl, to_resolve, results, err, resolve_tabl
       end
     elseif rbl.returncodes then
       for s, codes in pairs(rbl.returncodes) do
-        for _, v in ipairs(codes) do
-          if string.find(ipstr, '^' .. v .. '$') then
-            foundrc = true
-            insert_results(s)
-            break
-          end
+        local res = match(codes, ipstr, result, returncodes_maps[s])
+        if res then
+          foundrc = true
+          insert_results(s)
         end
       end
     end
@@ -436,6 +473,17 @@ local function gen_rbl_callback(rule)
     return true
   end
 
+  local function is_allowed(task, _)
+    if disabled_rbl_suffixes then
+      if disabled_rbl_suffixes:get_key(rule.rbl) then
+        lua_util.debugm(N, task, 'skip rbl check: %s; disabled by suffix', rule.rbl)
+        return false
+      end
+    end
+
+    return true
+  end
+
   local function check_required_symbols(task, _)
     if rule.require_symbols then
       return fun.all(function(sym)
@@ -562,7 +610,7 @@ local function gen_rbl_callback(rule)
       ignore_ip = rule.no_ip,
       need_images = rule.images,
       need_emails = false,
-      need_content = rule.content_urls or false,
+      need_content = rule.content_urls,
       esld_limit = esld_lim,
       no_cache = true,
     }
@@ -655,12 +703,19 @@ local function gen_rbl_callback(rule)
 
     local received_total = #received
     local check_conditions = gen_check_rcvd_conditions(rule, received_total)
+    local from_ip = task:get_from_ip()
 
     for pos, rh in ipairs(received) do
       if check_conditions(rh, pos) then
-        add_dns_request(task, rh.real_ip, false, true,
-            requests_table, 'received',
-            whitelist)
+        if rh.real_ip ~= from_ip or (rh.real_ip == from_ip and not rule.from) then
+          add_dns_request(task, rh.real_ip, false, true,
+              requests_table, 'received',
+              whitelist)
+        else
+          lua_util.debugm(N, task, 'rbl %s; skip check_received for %s:' ..
+              'Received IP same as From IP and will be checked only in check_from function',
+              rule.symbol, rh.real_ip)
+        end
       end
     end
 
@@ -797,6 +852,7 @@ local function gen_rbl_callback(rule)
 
   -- Create function pipeline depending on rbl settings
   local pipeline = {
+    is_allowed, -- check if rbl is allowed
     is_alive, -- check monitored status
     check_required_symbols -- if we have require_symbols then check those symbols
   }
@@ -809,7 +865,7 @@ local function gen_rbl_callback(rule)
     description[#description + 1] = 'user'
   end
 
-  if rule.exclude_local or rule.exclude_private_ips then
+  if rule.exclude_local then
     pipeline[#pipeline + 1] = check_local
     description[#description + 1] = 'local'
   end
@@ -858,6 +914,11 @@ local function gen_rbl_callback(rule)
     description[#description + 1] = 'selector'
   end
 
+  if not rule.returncodes_matcher then
+    rule.returncodes_matcher = 'equality'
+  end
+  local match = matchers[rule.returncodes_matcher]
+
   local callback_f = function(task)
     -- DNS requests to issue (might be hashed afterwards)
     local dns_req = {}
@@ -865,7 +926,7 @@ local function gen_rbl_callback(rule)
 
     local function gen_rbl_dns_callback(resolve_table_elt)
       return function(_, to_resolve, results, err)
-        rbl_dns_process(task, rule, to_resolve, results, err, resolve_table_elt)
+        rbl_dns_process(task, rule, to_resolve, results, err, resolve_table_elt, match)
       end
     end
 
@@ -937,7 +998,7 @@ local function gen_rbl_callback(rule)
         if req.resolve_ip then
           -- Deal with both ipv4 and ipv6
           -- Resolve names first
-          if r:resolve_a({
+          if (rule.ipv4 == nil or rule.ipv4) and r:resolve_a({
             task = task,
             name = req.n,
             callback = gen_rbl_ip_dns_callback(req),
@@ -945,7 +1006,7 @@ local function gen_rbl_callback(rule)
           }) then
             nresolved = nresolved + 1
           end
-          if r:resolve('aaaa', {
+          if (rule.ipv6 == nil or rule.ipv6) and r:resolve('aaaa', {
             task = task,
             name = req.n,
             callback = gen_rbl_ip_dns_callback(req),
@@ -972,6 +1033,12 @@ local function gen_rbl_callback(rule)
   return callback_f, string.format('checks: %s', table.concat(description, ','))
 end
 
+local map_match_types = {
+  glob = true,
+  radix = true,
+  regexp = true,
+}
+
 local function add_rbl(key, rbl, global_opts)
   if not rbl.symbol then
     rbl.symbol = key:upper()
@@ -1010,7 +1077,7 @@ local function add_rbl(key, rbl, global_opts)
             rbl.selector_flatten)
 
         if not sel then
-          rspamd_logger.errx('invalid selector for rbl rule %s: %s', key, selector)
+          rspamd_logger.errx(rspamd_config, 'invalid selector for rbl rule %s: %s', key, selector)
           return false
         end
 
@@ -1049,6 +1116,20 @@ local function add_rbl(key, rbl, global_opts)
         def_type, rbl.symbol)
   end
 
+  local match_type = rbl.returncodes_matcher
+  if match_type and rbl.returncodes and map_match_types[match_type] then
+    if not rbl.returncodes_maps then
+      rbl.returncodes_maps = {}
+    end
+    for label, v in pairs(rbl.returncodes) do
+      if type(v) ~= 'table' then
+        v = { v }
+      end
+      rbl.returncodes_maps[label] = lua_maps.map_add_from_ucl(v, match_type,
+          string.format('%s_%s RBL returncodes', label, rbl.symbol))
+    end
+  end
+
   if rbl.url_compose_map then
     local lua_urls_compose = require "lua_urls_compose"
     rbl.url_compose_map = lua_urls_compose.add_composition_map(rspamd_config, rbl.url_compose_map)
@@ -1059,11 +1140,11 @@ local function add_rbl(key, rbl, global_opts)
     end
   end
 
-  if not rbl.whitelist and global_opts.url_whitelist and
+  if not rbl.whitelist and not rbl.ignore_url_whitelist and (global_opts.url_whitelist or rbl.url_whitelist) and
       (rbl.urls or rbl.emails or rbl.dkim or rbl.replyto) and
       not (rbl.from or rbl.received) then
     local def_type = 'set'
-    rbl.whitelist = lua_maps.map_add_from_ucl(global_opts.url_whitelist, def_type,
+    rbl.whitelist = lua_maps.map_add_from_ucl(rbl.url_whitelist or global_opts.url_whitelist, def_type,
         'RBL url whitelist for ' .. rbl.symbol)
     rspamd_logger.infox(rspamd_config, 'added URL whitelist for RBL %s',
         rbl.symbol)
@@ -1254,6 +1335,11 @@ if type(opts.attached_maps) == 'table' then
   end
 end
 
+if opts.disabled_rbl_suffixes_map then
+  disabled_rbl_suffixes = lua_maps.map_add_from_ucl(opts.disabled_rbl_suffixes_map, 'set',
+      'Disabled suffixes for RBL')
+end
+
 for key, rbl in pairs(opts.rbls) do
   if type(rbl) ~= 'table' or rbl.disabled == true or rbl.enabled == false then
     rspamd_logger.infox(rspamd_config, 'disable rbl "%s"', key)
@@ -1284,7 +1370,7 @@ for key, rbl in pairs(opts.rbls) do
       rspamd_logger.errx(rspamd_config, 'invalid config for %s: %s, RBL is DISABLED',
           key, err)
     else
-      res = rbl_common.convert_checks(res)
+      res = rbl_common.convert_checks(res, rbl.symbol or key:upper())
       -- Aliases
       if res.return_codes then
         res.returncodes = res.return_codes
